/* The Monitor
 * Copyright (C) 1998 Oskar Liljeblad
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */

#include "monitor.h"
#include "util.h"
#include "ui.h"
#include "common.h"

struct arg_options ui_opts[] = { { NULL } };
char *ui_base_name = "mon";
int running;

int
initialize_ui (argc, argv)
  int *argc;
  char ***argv;
{
	return TRUE;
}

int
enter_ui_mainloop (opts)
  struct arg_options *opts;
{
	Watch *watch;
	fd_set fds;
	int maxfd;

	running = TRUE;
	while(running) {

		/* initialize file descriptor set */
		FD_ZERO(&fds);
		maxfd = 0;
		watch = first_watch;
		while (watch) {
			if (watch->fd != -1) {
				FD_SET(watch->fd, &fds);
				maxfd = MAX(maxfd, watch->fd);
			}
			watch = watch->next;
		}

		/* wait for file descriptors */
		if (select(maxfd+1, &fds, NULL, NULL, NULL) == -1 && errno != EINTR) {
			perror(ui_base_name);
			return FALSE;
		}
	
		/* check and update triggered fds */
		watch = first_watch;
		while (watch) {
			if (watch->fd != -1 && FD_ISSET(watch->fd, &fds)) {
				if (!update_watch(watch))
					return FALSE;
			}
			watch = watch->next;
		}
	}

	return TRUE;
}

void
exit_ui (void)
{
	running = FALSE;
}

int 
monitor_file_event (watch)
  Watch *watch;
{
	return TRUE;
}

void
unmonitor_file_event (watch)
  Watch *watch;
{
	/* no operation necessary */
}

int
monitor_timer (watch)
  Watch *watch;
{
	/* no operation necessary */
	return TRUE;
}

void
unmonitor_timer (watch)
  Watch *watch;
{
	/* no operation necessary */
}

int
ui_parse_args_callback(str, opt, opts)
  char *str;
  int opt;
  struct arg_options *opts;
{
	if (opts) {
		if (str) {
			Watch *watch;
			watch = new_watch(str, 0);
			if (!watch)
				return FALSE;
		}
	}

	return TRUE;
}

int
ui_print_line (watch, str, len, type)
  Watch *watch;
  char *str;
  unsigned int len;
  int type;
{
	/* if all wasn't writen, well, do nothing about it right now. FIXME: do xwrite */

	if (write(STDOUT_FILENO, str, len) == -1) {
		perror(ui_base_name);	
		return FALSE;
	}

	return TRUE;
}

void ui_close_watch(watch)
  Watch *watch;
{
	/* no operation necessary */
}
