/* The Monitor
 * Copyright (C) 1998 Oskar Liljeblad
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */

#ifndef __MONITOR_MONITOR_H__
#define __MONITOR_MONITOR_H__

#ifdef HAVE_CONFIG_H
#  include "../config.h"
#endif

#ifndef _GNU_SOURCE
#  define _GNU_SOURCE
#endif

#if HAVE_UNISTD_H
#  include <sys/types.h>
#  include <unistd.h>
#else
#  ifndef STDIN_FILENO			/* shameless unistd.h redefinitions */
#    define STDIN_FILENO 0
#  endif
#  ifndef STDOUT_FILENO
#    define STDOUT_FILENO 1
#  endif
#  ifndef STDERR_FILENO
#    define STDERR_FILENO 2
#  endif
#endif

#if HAVE_FCNTL_H
#  include <fcntl.h>
#endif

#if TIME_WITH_SYS_TIME
#  include <sys/time.h>
#  include <time.h>
#else
#  if HAVE_SYS_TIME_H
#    include <sys/time.h>
#  else
#    include <time.h>
#  endif
#endif

#if HAVE_SYS_STAT_H
#  include <sys/stat.h>
#endif

#if STDC_HEADERS
#  include <stdlib.h>
#  include <stdarg.h>
#  include <string.h>
#  include <stdio.h>
#  include <ctype.h>
#  include <errno.h>
#  include <signal.h>
#else
#  if HAVE_VARARGS_H
#    include <varargs.h>
#  endif
#  if !HAVE_MEMCPY
#    define memcpy(d,s,n) bcopy((s), (d), (n))
#  endif
#  if !HAVE_STRDUP
char *strdup(char *);
#  endif
#endif

#include "util.h"

typedef struct _Watch Watch;
#define WATCH(x) ((Watch *) x)

enum {
	OPT_TIMESTAMP,
	OPT_NO_TIMESTAMP,
	OPT_SYSLOG,
	OPT_NO_SYSLOG,
	OPT_DELAY,
//	OPT_CONFIG,
	OPT_HELP,
	OPT_VERSION,
	LAST_OPT
};

struct _Watch {
	Watch *next;
	unsigned int type;			/* type of watch */
	int fd;						/* file descriptor */
	unsigned int delay;			/* delay between reads */
	unsigned int flags;			/* see below */
	char *pending;				/* pending string (to be outputted) */
	unsigned int pending_len;	/* length of pending string */
	char *timestamp;			/* timestamp string */
	char *filename;
};

// watch flags
#define WATCH_FLAG_TIMESTAMP	1
#define WATCH_FLAG_SYSLOG		2

// watch types
#define WATCH_REGULAR 1
#define WATCH_FIFO 2
#define WATCH_SOCKET 3

#define WATCH_TYPE_TIMER 1
#define WATCH_TYPE_EVENT 2

Watch *new_watch(char *, int);
void close_watches(void);
int update_watch (Watch *);
void close_watch (Watch *);

extern Watch *first_watch;

#endif /* __MONITOR_MONITOR_H__ */
