/* The Monitor
 * Copyright (C) 1998 Oskar Liljeblad
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */

#include "monitor.h"		/* necessary for <str*.h> etc */
#include "util.h"
#include "common.h"
#include "ui.h"			/* necessary for ui_base_name */

/* xread - reads exactly (size) bytes, retrying until enough bytes have been read.
 * this is unused at the moment
 */
/*
int
xread(fd, buf, size)
  int fd;
  void *buf;
  size_t size;
{
	int c;
	size_t left;

	left = size;
	while (left > 0) {
		c = read(fd, buf, left);
		if (c == -1 || c == 0) return FALSE;
		left -= c;
		buf = (void *) (((char *) buf)+c);
	}
	return TRUE;
}
*/

#ifdef STDC_HEADERS
void
printerror(char *firststr, ...)
#else
void
printerror(va_alist)
  va_dcl
#endif
{
	char *str;
	va_list ap;

	fprintf(stderr, "%s: ",ui_base_name);

#ifdef STDC_HEADERS
	fprintf(stderr, "%s", firststr);
	va_start(ap, firststr);
#else
	va_start(ap);
#endif

	while ((str = va_arg(ap, char *))) {
		fprintf(stderr, "%s", str);
	}

	fputc('\n', stderr);

	va_end(ap);
}

void
syslog_time_to_tm (str, tm)
  char *str;
  struct tm *tm;
{
	register char ch1,ch2,ch3;

	memset(tm, 0, sizeof(struct tm));

	tm->tm_hour = (str[7]-'0')*10 + (str[8]-'0');
	tm->tm_min = (str[10]-'0')*10 + (str[11]-'0');
	tm->tm_sec = (str[13]-'0')*10 + (str[14]-'0');
	tm->tm_mday = (str[4] == ' ' ? 0 : (str[4]-'0'))*10 + (str[5]-'0');
	tm->tm_year = 0;
	tm->tm_wday = 0;
	tm->tm_yday = 0;
	tm->tm_isdst = 0;

	/* extremely optimized month testing scheme follows :) */

	ch1 = str[0]; ch2 = str[1]; ch3 = str[2];
											// get: Jan,Feb,Mar,Apr,May,Jun,Jul,Aug,Sep,Oct,Nov,Dec
	if (ch1 == 'J')							//   get: Jan,Jun,Jul
		if (ch2 == 'u')						//     get: Jun,Jul
			if (ch3 == 'n') tm->tm_mon = 5;	//       get: Jun
			else tm->tm_mon = 6;			//       rem: Jul
		else tm->tm_mon = 0;				//     rem: Jan
	else if (ch2 == 'e')					//   get: Feb,Sep,Dec
		if (ch1 == 'F') tm->tm_mon = 1;		//     get: Feb
		else if (ch1 == 'S') tm->tm_mon = 8;//     get: Sep
		else tm->tm_mon = 11;				//     rem: Dec
	else if (ch1 == 'M')					//   get: Mar,May
		if (ch3 == 'r') tm->tm_mon = 2;		//     get: Mar
		else tm->tm_mon = 4;				//     rem: May
	else if (ch1 == 'A')					//   get: Apr,Aug
		if (ch2 == 'p') tm->tm_mon = 3;		//     get: Apr
		else tm->tm_mon = 7;				//     rem: Aug
	else if (ch1 == 'O') tm->tm_mon = 9;	//   get: Oct
	else tm->tm_mon = 10;					//   rem: Nov

	/* total character tests for
	 * Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec
	 *   2   3   4   5   4   3   3   5   4   5   5   4
     * best case: 2   worst case: 5   average case:  3.9
	 */
}

/* get_whole_line:
 *   returns a pointer to string in argument, if there is a full line in the
 *   argument (terminated by LF). the argument is updated to point to after
 *   the line returned. if there isn't a full line, NULL is returned.
 */
char *
get_whole_line(inputline)
  char **inputline;
{
	int c;
	char *outputline = *inputline;

	for(c = 0; (*inputline)[c] && (*inputline)[c] != '\n'; c++);
	if ((*inputline)[c] == '\n') {
		*inputline = &(*inputline)[c+1];
		return outputline;
	}

	return NULL;
}

/* arg_parser - when the application needs to be notified for each argument
 *
 */
int arg_parser (argcnt, argvec, opts, callback)
  int argcnt;
  char **argvec;
  struct arg_options *opts;
  arg_parser_callback callback;
{
	int c;
	int pending_arg;

	pending_arg = -1;
	for(c=0; c<argcnt; c++) {
		if (pending_arg >= 0) {
			opts[pending_arg].string = argvec[c];
			opts[pending_arg].exists = TRUE;
			if (!callback(NULL, pending_arg, opts))
				return FALSE;
			pending_arg = -1;
		} else if (pending_arg == -1) {
			if (argvec[c][0] == '-' && argvec[c][1] == '-') {
				if (!argvec[c][2]) {
					pending_arg = -2;
				} else {
					register int c2;
					for(c2 = 0; opts[c2].long_name; c2++) {
						register int len;
						len = strlen(opts[c2].long_name);
						if (!strncmp(&argvec[c][2], opts[c2].long_name, len)) {
							if  (!argvec[c][len+2]) {
								opts[c2].exists = TRUE;
								if (opts[c2].flags == ARG_REQUIRED)
									pending_arg = c2;
								else {
									opts[c2].string = NULL;
									if (!callback(NULL, c2, opts))
										return FALSE;
								}
								goto outter;
							} else if (argvec[c][len+2] == '=') {
								if (opts[c2].flags == ARG_NONE) {
									char *str=NULL;
									asprintf(&str, "option `--%s' doesn't allow an argument",opts[c2].long_name);
									if (str) {
										callback(str, 0, NULL);
										free(str);
									} else
										printerror("Memory exhausted", NULL);
									return FALSE;
								} else {
									opts[c2].string = &argvec[c][len+3];
									opts[c2].exists = TRUE;
									if (!callback(NULL, c2, opts))
										return FALSE;
								}
								goto outter;
							}
						}
					}
					{
						char *str=NULL;
						asprintf(&str, "unrecognized option `%s'", argvec[c]);
						if (str) {
							callback(str, 0, NULL);
							free(str);
						} else
							printerror("Memory exhausted", NULL);
						return FALSE;
					}
				}
			} else if (argvec[c][0] == '-' && argvec[c][1]) {
				register int c3;
				for (c3 = 1; argvec[c][c3]; c3++) {
					register int c2;
					int found = FALSE;
					for (c2 = 0; opts[c2].long_name; c2++) {
						if (opts[c2].abbreviation == argvec[c][c3]) {
							opts[c2].exists = TRUE;
							if (opts[c2].flags == ARG_REQUIRED)
								pending_arg = c2;
							else if (opts[c2].flags == ARG_OPTIONAL)
								opts[c2].string = NULL;
							found = TRUE;
							if (!callback(NULL, c2, opts))
								return FALSE;
						}
					}
					if (!found) {
						char *str=NULL;
						asprintf(&str, "invalid option -- %c", argvec[c][c3]);
						if (str) {
							callback(str, 0, NULL);
							free(str);
						} else
							printerror("Memory exhausted", NULL);
						return FALSE;
					}
				}
			} else {
				if (!callback(argvec[c], 0, opts))
					return FALSE;
			}
		}
outter:
	}

	if (pending_arg >= 0) {
		char *str=NULL;
		asprintf(&str, "option `--%s' requires an argument", opts[pending_arg].long_name);
		if (str) {
			callback(str, 0, NULL);
			free(str);
		} else
			printerror("Memory exhausted", NULL);
		return FALSE;
	}

	return TRUE;
}
